function [ n,q,leave,emission ] = Link1_L( n,q,enter,green_time,cycle_time,C_down,para, scale )
%LINK1 Link1 is the model for links that have only one downstream with only one lane
% q is a scalar denotes the queue on the only lane
% enter is the total entering flow vector for time k-1,k, veh/h, is calculated in advance
% the green_time is the green light time in a cycle, with the unit as second for convenience
% para=[link_length; vehicle_length; free_flow_speed; [turning ratio]';[saturating_flow_rate]']
% C_down are downstream available capacities
% the outputs are the number of vehicles, the queue length, and the
% available capacity of the link
% scale =1;
cd = cycle_time; % the unit is second
l_link = para(1); % link length
l_veh = para(2); % vehicle length
v_free = para(3)/3.6; % free flow speed, m/s
v_low = para(4)/3.6; % low speed, m/s
N_lane = para(5); % number of lanes of link
beta = para(6); % turning ratio ( is 100%)
mu = para(7); % saturating flow rate for turning
q_sum=sum(q); % the total queue length on the link
C = floor(N_lane*l_link/l_veh)*scale; % the maximum capacity of the link

acc=2; %accelaration m/s2, speed up
dec=-2; %accelaration m/s2, slow down

tao = floor(((C-q_sum)*l_veh/scale)/(N_lane*v_free*cd)); % delayed cycle
gama = rem(l_veh*(C-q_sum)/scale/(N_lane*v_free), cd); % delayed seconds
if tao<0
    gama=0; tao=0;
end
arriv = (cd-gama)/cd*enter(end-tao) + gama/cd*enter(end-tao-1); % arriving flow at current cycle
arriv_1 = arriv*beta; % arriving flow
% arriv = enter(end);
% arriv_1 = enter(end)*beta; % arriving flow for turning left
%% calculating the leaving flow
Vector1 = [mu*green_time/cd, q/cd*3600+arriv_1, C_down/cd*3600]; % lane 1, leaving flow

leave = min(Vector1);
%% state evolution

n_ = n + (enter(end) - leave)*cd/3600; % updated total number of vehicles
q = q + (arriv_1 - leave)*cd/3600; % updated queue length
q_sum=q;
%% calculate the emission
% judge the current situation: saturated, over-saturated, unsaturated scenario
if q_sum <= beta*mu*green_time/3600 && beta*mu*green_time/3600<= arriv*cd/3600+q_sum
%     N_free = max(0, n-cd/3600*arriv-q_sum);
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = cd/3600*arriv+q_sum-beta*mu*green_time/3600; % the vehicles that cannot leave
    N_idl_2 = beta*mu*green_time/3600-q_sum; % the vehicles that can leave
    N_idl_3 = 0;
    N_idl_4 = q_sum; % the vehicles that can leave
    N_dec = cd/3600*arriv;
    N_acc = beta*mu*green_time/3600;
    N_non = 0;
    t_free = cd;
    t_idl_1 = cd - (v_low-v_free)/dec;
    t_idl_2 = cd/scale - (v_low-v_free)/dec - (v_free-v_low)/acc;
    t_idl_3 = 0;
    t_idl_4 = cd/scale - (v_free-v_low)/acc;
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = 0;
elseif beta*mu*green_time/3600<q_sum
%     N_free = max(0, n-cd/3600*arriv-q_sum);
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = cd/3600*arriv; % the vehicles that cannot leave
    N_idl_2 = 0;
    N_idl_3 = q_sum-beta*mu*green_time/3600; % the vehicles that cannot leave
    N_idl_4 = beta*mu*green_time/3600; % the vehicles that can leave
    N_dec = cd/3600*arriv;
    N_acc = beta*mu*green_time/3600;
    N_non = 0;
    t_free = cd;
    t_idl_1 = cd - (v_low-v_free)/dec;
    t_idl_2 = 0;
    t_idl_3 = cd;
    t_idl_4 = cd/scale - (v_free-v_low)/acc;
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = 0;
elseif arriv*cd/3600+q_sum<beta*mu*green_time/3600
    Gs = (cd/3600*arriv+q_sum-green_time/3600*arriv)/(mu-arriv)*3600;
    Gd = (green_time/3600*mu-cd/3600*arriv-q_sum)/(mu-arriv)*3600;
%     N_free = max(0, n-cd/3600*arriv-q_sum);
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = 0;
    N_idl_2 = Gs*mu/3600-q_sum; % the vehicles that can leave
    N_idl_3 = 0;
    N_idl_4 = q_sum; % the vehicles that can leave
    N_dec = (cd-Gd)*arriv/3600;
    N_acc =Gs*mu/3600;
    N_non = Gd*arriv/3600;
    t_free = cd;
    t_idl_1 = 0;
    t_idl_2 = cd/scale-Gd/scale-(v_low-v_free)/dec-(v_free-v_low)/acc;
    t_idl_3 = 0;
    t_idl_4 = cd/scale - Gd/scale - (v_free-v_low)/acc;
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = Gd/scale;
else
    error('Something goes wrong with the situation judging');
end
% calculate the emissions of all the vehicles
% [CO, HC, NOx, CO2, Fc] = VTmicro(v, a, 2);
e_free = 0.0032;

e_idl = 0.0016;

e_dec = 0.0023;

e_acc = 0.0118;

emission = (N_free*t_free+N_non*t_non).*e_free+(N_idl_1*t_idl_1+N_idl_2*t_idl_2+N_idl_3*t_idl_3+N_idl_4*t_idl_4)*e_idl...
    + N_dec*t_dec*e_dec + N_acc*t_acc*e_acc;
n=n_;

end


