function [ n,q,leave,emission,emis_num_time] = Link2( n,q,enter,green_time,cycle_time,C_down,para )
%LINK2 is the link model for the links that have two downstreams
%   these links should have 2 lanes, each of which going to the
%   corresponding downstreams, and they are numbered as 1, 2, with the order left, straight, and right; cycle phases are still 2
% q=[q1; q2], q1 denotes the queue on lane 1, q2 on lane 2; n is the total number of vehicles on the link;
% enter is the total entering flow vector for time k-1,k, veh/h, is calculated in advance
% the green_time is the green light time in a cycle, with the unit as second for convenience
% para=[link_length; vehicle_length; free_flow_speed; [turning ratio]';[saturating_flow_rate]']
% C_down are downstream available capacities
% the outputs are the number of vehicles, the queue lengths, and the
% available capacity of the link
cd = cycle_time; % the unit is second
l_link = para(1); % link length
l_veh = para(2); % vehicle length
v_free = para(3)/3.6; % free flow speed, m/s
v_low = para(4)/3.6; % low speed, m/s
N_lane = para(5); % number of lanes of link
beta = para(6:7); % turning ratio for 1, 2
mu = para(8:9); % saturating flow rate for turning 1,2  
q_sum=sum(q); % the total queue length on the link
C = floor(N_lane*l_link/l_veh); % the maximum capacity of the link
acc=2; %accelaration m/s2, speed up
dec=-2; %accelaration m/s2, slow down
tao = floor(((C-q_sum)*l_veh)/(N_lane*v_free*cd)); % delayed cycle
gama = rem(l_veh*(C-q_sum)/(N_lane*v_free), cd); % delayed seconds
if tao<0
    gama=0; tao=0;
elseif tao>1
    tao=1;gama=cd;
end

arriv = (cd-gama)/cd*enter(end-tao) + gama/cd*enter(end-tao-1); % arriving flow at current cycle
arriv_1 = arriv*beta(1); % arriving flow for turning left
arriv_2 = arriv*beta(2); % arriving flow for going straight
% arriv = enter(end);
% arriv_1 = enter(end)*beta(1); % arriving flow for turning left
% arriv_2 = enter(end)*beta(2); % arriving flow for going straight
%% calculating the leaving flow
Vector1 = [mu(1)*green_time/cd, q(1)/cd*3600+arriv_1, beta(1)*C_down(1)/cd*3600]; % lane 1, leaving flow
Vector2 = [mu(2)*green_time/cd, q(2)/cd*3600+arriv_2, beta(2)*C_down(2)/cd*3600]; % lane 2, leaving flow

leave(1) = min(Vector1);
leave(2) = min(Vector2);
%% state evolution


leave_total = leave(1) + leave(2);

n_ = n + (enter(end) - leave_total)*cd/3600; % updated total number of vehicles
q(1) = q(1) + (arriv_1 - leave(1))*cd/3600; % updated queue length
q(2) = q(2) + (arriv_2 - leave(2))*cd/3600;
% q_sum=sum(q);
%% calculate the emission
% judge the current situation: saturated, over-saturated, unsaturated scenario
if q_sum <= sum(mu)*green_time/3600 && sum(mu)*green_time/3600<= arriv*cd/3600+q_sum
%     N_free = max(-1, n-cd/3600*arriv-q_sum);
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = cd/3600*arriv+q_sum-sum(mu)*green_time/3600;
    N_idl_2 = sum(mu)*green_time/3600-q_sum;
    N_idl_3 = 0;
    N_idl_4 = q_sum;
    N_dec = cd/3600*arriv;
    N_acc = sum(mu)*green_time/3600;
    N_non = 0;
    t_free = cd;
    t_idl_1 = cd - (v_low-v_free)/dec;
    t_idl_2 = cd - (v_low-v_free)/dec - (v_free-v_low)/acc;
    t_idl_3 = 0;
    t_idl_4 = cd - (v_free-v_low)/acc;
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = 0;
elseif sum(mu)*green_time/3600<q_sum
%     N_free = max(-1, n-cd/3600*arriv-q_sum);
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = cd/3600*arriv;
    N_idl_2 = 0;
    N_idl_3 = q_sum-sum(mu)*green_time/3600;
    N_idl_4 = sum(mu)*green_time/3600;
    N_dec = cd/3600*arriv;
    N_acc = sum(mu)*green_time/3600;
    N_non = 0;
    t_free = cd;
    t_idl_1 = cd - (v_low-v_free)/dec;
    t_idl_2 = 0;
    t_idl_3 = cd;
    t_idl_4 = cd - (v_free-v_low)/acc;
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = 0;
elseif arriv*cd/3600+q_sum<sum(mu)*green_time/3600
    Gs = (cd/3600*arriv+q_sum-green_time/3600*arriv)/(sum(mu)-arriv)*3600;
    Gd = (green_time/3600*sum(mu)-cd/3600*arriv-q_sum)/(sum(mu)-arriv)*3600;
%     N_free = max(-1, n-cd/3600*arriv-q_sum);
    
    N_free = n+enter(end)*cd/3600-cd/3600*arriv-q_sum;
    N_idl_1 = 0;
    N_idl_2 = Gs*sum(mu)/3600-q_sum;
    N_idl_3 = 0;
    N_idl_4 = q_sum;
    N_dec = (cd-Gd)*arriv/3600;
    N_acc =Gs*sum(mu)/3600;
    N_non = Gd*arriv/3600;
    t_free = cd;
    t_idl_1 = 0;
    t_idl_2 = max(0, cd-Gd-(v_low-v_free)/dec-(v_free-v_low)/acc);
    t_idl_3 = 0;
    t_idl_4 = max(0, cd - Gd - (v_free-v_low)/acc);
    t_dec = (v_low-v_free)/dec;
    t_acc = (v_free-v_low)/acc;
    t_non = Gd;
else
    error('Something goes wrong with the situation judging');
end
if N_free<0
    N_free=0;
    warning('N_free is 0');
end
% calculate the emissions of all the vehicles
% [CO, HC, NOx, CO2, Fc] = VTmicro(v, a, 2);
e_free = 0.0032;

e_idl = 0.0016;

e_dec = 0.0023;

e_acc = 0.0118;

emission = (N_free*t_free+N_non*t_non).*e_free+(N_idl_1*t_idl_1+N_idl_2*t_idl_2+N_idl_3*t_idl_3+N_idl_4*t_idl_4)*e_idl...
    + N_dec*t_dec*e_dec + N_acc*t_acc*e_acc;
emis_num_time=[N_free N_idl_1+N_idl_2+N_idl_3+N_idl_4 N_dec N_acc t_free t_idl_1+t_idl_2+t_idl_3+t_idl_4 t_dec t_acc];
n=n_;

end

