clear; clc; close all
%% Simulation of the traffic network for 1 hour; S-model, cycle=1min
%% define the demands of the source links
tic
addpath('.\Source_1');
addpath('.\Source_oneday');
scale = 5;  % time scale
%% define the parameters of all the links
% parameters=[5.6986,1824,45.27,621.7,1150.58,725.329,771.39,929,863,1217]';
parameters=[4.1357; 2354.2; 43.51; 671.61; 1104.24; 977.80; 887.358; 798.51; 809.71; 1029.577];
% parameters=[6.236; 2356.323; 40.153; 537.598; 1109.963; 964.947; 837.606; 689.730; 855.036; 1016.662];
% parameters=[5 2000 60 700 1000 800 750 850 900 1200]'; % default value
l_veh=parameters(1); % default: 5m
sat=parameters(2); % saturating flow, default 2000
v_free=parameters(3); % default: 60km/h
v_idl=4;
l_1=parameters(4); % default: 700m
l_2=parameters(5);% default: 1000m
l_3=parameters(6); % default: 800m
l_4=parameters(7); % default: 750m
l_5=parameters(8); % default: 850m
l_6=parameters(9); % default: 900m
l_7=parameters(10); % default: 1200m
% parameter vector [link_length vehicle_lengt free_flow_speed low_speed turning_ratio[left straight right] saturating_flow[left straight right]]
Link_AB=[l_1, l_veh, v_free, v_idl, 2, [0.2 0.8], [sat, sat]]; %parameters of link(A,B), Link_2
Link_BA=[l_1, l_veh, v_free, v_idl, 3, [0.7 0.15 0.15], [sat, sat, sat]]; %parameters of link(A,B), Link_3, left turning ratio is small
Link_BC=[l_1, l_veh, v_free, v_idl, 3, [0.15 0.15 0.7], [sat, sat, sat]]; %parameters of link(B,C), Link_2
Link_CB=[l_1, l_veh, v_free, v_idl, 2, [0.8 0.2], [sat, sat]]; %parameters of link(A,B), Link_2
Link_AD=[l_3, l_veh, v_free, v_idl, 2, [0.4 0.2 0.4] [sat sat sat]];
Link_BE=[l_5, l_veh, v_free, v_idl, 3, [0.25 0.5 0.25], [sat sat sat]];
Link_CF=[l_1, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DE=[l_4, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]]; % Link_2
Link_ED=[l_4, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]]; %LInk_2
Link_EF=[l_3, l_veh, v_free, v_idl, 1, 1, sat]; %Link_1
Link_FE=[l_3, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DG=[l_6, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]];
Link_EH=[l_2, l_veh, v_free, v_idl, 3, [0.40 0.20 0.40], [sat sat sat]];
Link_FI=[l_7, l_veh, v_free, v_idl, 2, [0.6 0.4], [sat sat]];
Link_GH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_HG=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat]; %Link_1
Link_HI=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat];
Link_IH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_1A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_2A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_3C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_4C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_5I=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_6H=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_7G=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_8D=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
%% define the initial state of the links
% Capacities
scale1 = scale;
% initial entering
enter_AB_=0;enter_BC_=0;enter_BA_=0;enter_CB_=0;enter_AD_=0;enter_BE_=0;enter_CF_=0;enter_DE_=0;
enter_ED_=0;enter_EF_=0;enter_FE_=0;enter_DG_=0;enter_EH_=0;enter_FI_=0;enter_GH_=0;enter_HG_=0;
enter_HI_=0;enter_IH_=0;
enter_AB__=0;enter_BC__=0;enter_BA__=0;enter_CB__=0;enter_AD__=0;enter_BE__=0;enter_CF__=0;enter_DE__=0;
enter_ED__=0;enter_EF__=0;enter_FE__=0;enter_DG__=0;enter_EH__=0;enter_FI__=0;enter_GH__=0;enter_HG__=0;
enter_HI__=0;enter_IH__=0;
% number of vehicles and queues
n_AB=0;q_AB=[0 0];
n_BA=0;q_BA=[0 0 0];
n_BC=0;q_BC=[0 0 0];
n_CB=0;q_CB=[0 0];
n_AD=0;q_AD=[0 0 0];
n_BE=0;q_BE=[0 0 0];
n_CF=0;q_CF=[0 0];
n_DE=0;q_DE=[0 0];
n_ED=0;q_ED=[0 0];
n_EF=0;q_EF=0;
n_FE=0;q_FE=[0 0];
n_DG=0;q_DG=[0 0];
n_EH=0;q_EH=[0 0 0];
n_FI=0;q_FI=[0 0];
n_GH=0;q_GH=[0 0];
n_HG=0;q_HG=0;
n_HI=0;q_HI=0;
n_IH=0;q_IH=[0 0];
n_1A=0; q_1A=[0 0];
n_2A=0; q_2A=[0 0];
n_3C=0; q_3C=[0 0];
n_4C=0; q_4C=[0 0];
n_5I=0; q_5I=[0 0];
n_6H=0; q_6H=[0 0];
n_7G=0; q_7G=[0 0];
n_8D=0; q_8D=[0 0];
% control variable
hour =24;
N=hour*60/scale; cycle=scale*60;
Emission_total_L = NaN(1, N);
green_time=scale*30*ones(8,N); % 9 intersections, for 60 minutes, 1st row for A, ..., 8th row for I; unit is second
N_total_L = zeros(1, N);
Q_AB_L=zeros(1, N);
Q_BA_L=zeros(1, N);
Q_BC_L=zeros(1, N);
Q_CB_L=zeros(1, N);
Q_AD_L=zeros(1, N);
Q_BE_L=zeros(1, N);
Q_CF_L=zeros(1, N);
Q_DE_L=zeros(1, N);
Q_ED_L=zeros(1, N);
Q_EF_L=zeros(1, N);
Q_FE_L=zeros(1, N);
Q_DG_L=zeros(1, N);
Q_EH_L=zeros(1, N);
Q_FI_L=zeros(1, N);
Q_GH_L=zeros(1, N);
Q_HG_L=zeros(1, N);
Q_HI_L=zeros(1, N);
Q_IH_L=zeros(1, N);
N_AB_L=zeros(1, N);
N_BA_L=zeros(1, N);
N_BC_L=zeros(1, N);
N_CB_L=zeros(1, N);
N_AD_L=zeros(1, N);
N_BE_L=zeros(1, N);
N_CF_L=zeros(1, N);
N_DE_L=zeros(1, N);
N_ED_L=zeros(1, N);
N_EF_L=zeros(1, N);
N_FE_L=zeros(1, N);
N_DG_L=zeros(1, N);
N_EH_L=zeros(1, N);
N_FI_L=zeros(1, N);
N_GH_L=zeros(1, N);
N_HG_L=zeros(1, N);
N_HI_L=zeros(1, N);
N_IH_L=zeros(1, N);
N_1A=NaN(1, N);
N_2A=NaN(1, N);
N_3C=NaN(1, N);
N_4C=NaN(1, N);
N_5I=NaN(1, N);
N_6H=NaN(1, N);
N_7G=NaN(1, N);
N_8D=NaN(1, N);
Emission_AB_L=NaN(1, N);
Emission_BA_L=NaN(1, N);
Emission_BC_L=NaN(1, N);
Emission_CB_L=NaN(1, N);
Emission_AD_L=NaN(1, N);
Emission_BE_L=NaN(1, N);
Emission_CF_L=NaN(1, N);
Emission_DE_L=NaN(1, N);
Emission_ED_L=NaN(1, N);
Emission_EF_L=NaN(1, N);
Emission_FE_L=NaN(1, N);
Emission_DG_L=NaN(1, N);
Emission_EH_L=NaN(1, N);
Emission_FI_L=NaN(1, N);
Emission_GH_L=NaN(1, N);
Emission_HG_L=NaN(1, N);
Emission_HI_L=NaN(1, N);
Emission_IH_L=NaN(1, N);
%% Simulation for one hour
for k=1:N
    C_AB=floor(Link_AB(1)*Link_AB(5)/Link_AB(2))*scale1;
    C_BA=floor(Link_BA(1)*Link_BA(5)/Link_BA(2))*scale1;
    C_BC=floor(Link_BC(1)*Link_BC(5)/Link_BC(2))*scale1;
    C_CB=floor(Link_CB(1)*Link_CB(5)/Link_CB(2))*scale1;
    C_AD=floor(Link_AD(1)*Link_AD(5)/Link_AD(2))*scale1;
    C_BE=floor(Link_BE(1)*Link_BE(5)/Link_BE(2))*scale1;
    C_CF=floor(Link_CF(1)*Link_CF(5)/Link_CF(2))*scale1;
    C_DE=floor(Link_DE(1)*Link_DE(5)/Link_DE(2))*scale1;
    C_ED=floor(Link_ED(1)*Link_ED(5)/Link_ED(2))*scale1;
    C_EF=floor(Link_EF(1)*Link_EF(5)/Link_EF(2))*scale1;
    C_FE=floor(Link_FE(1)*Link_FE(5)/Link_FE(2))*scale1;
    C_DG=floor(Link_DG(1)*Link_DG(5)/Link_DG(2))*scale1;
    C_EH=floor(Link_EH(1)*Link_EH(5)/Link_EH(2))*scale1;
    C_FI=floor(Link_FI(1)*Link_FI(5)/Link_FI(2))*scale1;
    C_GH=floor(Link_GH(1)*Link_GH(5)/Link_GH(2))*scale1;
    C_HG=floor(Link_HG(1)*Link_HG(5)/Link_HG(2))*scale1;
    C_HI=floor(Link_HI(1)*Link_HI(5)/Link_HI(2))*scale1;
    C_IH=floor(Link_IH(1)*Link_IH(5)/Link_IH(2))*scale1;
    [enter_AB_1, enter_AD_1, n_1A, q_1A, emission_1A_L] = Link_S1nLD(k, n_1A, q_1A, [C_AB, C_AD], green_time(1,k), cycle, Link_1A, scale); % entering flow from the source
    [enter_AB_2, enter_AD_2, n_2A, q_2A, emission_2A_L] = Link_S2nLD(k, n_2A, q_2A, [C_AB, C_AD], cycle-green_time(1,k), cycle, Link_2A, scale);
    [enter_CF_3, enter_CB_3, n_3C, q_3C, emission_3C_L] = Link_S3nLD(k, n_3C, q_3C, [C_CF, C_CB], cycle-green_time(2,k), cycle, Link_3C, scale);
    [enter_CF_4, enter_CB_4, n_4C, q_4C, emission_4C_L] = Link_S4nLD(k, n_4C, q_4C, [C_CF, C_CB], green_time(2,k), cycle, Link_4C, scale);
    [enter_IH_5, n_5I, q_5I, emission_5I_L] = Link_S5nLD(k, n_5I, q_5I, C_IH, green_time(8,k), cycle, Link_5I, scale);
    [enter_HG_6, enter_HI_6, n_6H, q_6H, emission_6H_L] = Link_S6nLD(k, n_6H, q_6H, [C_HG, C_HI], cycle-green_time(7,k), cycle, Link_6H, scale);
    [enter_GH_7, n_7G, q_7G, emission_7G_L] = Link_S7nLD(k, n_7G, q_7G, C_GH, cycle-green_time(6,k), cycle, Link_7G, scale);
    [enter_DE_8, enter_DG_8, n_8D, q_8D, emission_8D_L] = Link_S8nLD(k, n_8D, q_8D, [C_DE, C_DG], green_time(3, k), cycle, Link_8D, scale);
    % simulate from the source links; leave is a vector: leaving to corresponding downstream links
    %% link(A,B), B does not need control
    enter_AB = enter_AB_1+enter_AB_2;
    [n_AB, q_AB, leave_AB, emission_AB_L] = Link2_L(n_AB, q_AB, [enter_AB__, enter_AB_, enter_AB], cycle, cycle, [C_BC C_BE], Link_AB, scale);%update the state
    leave_AB_C = leave_AB(1); leave_AB_E = leave_AB(2); 
    C_AB=C_AB-n_AB;
    enter_AB__= enter_AB_; enter_AB_= enter_AB; % update the enter of previous step, B does not need control
    %% link(C,B), B does not need control
    enter_CB = enter_CB_3 + enter_CB_4;
    [n_CB, q_CB,  leave_CB, emission_CB_L] = Link2_L(n_CB, q_CB, [enter_CB__, enter_CB_, enter_CB], cycle, cycle, [C_BE, C_BA], Link_CB, scale);
    leave_CB_E = leave_CB(1); leave_CB_A = leave_CB(2);
    C_CB=C_CB-n_CB;
    enter_CB__ = enter_CB_; enter_CB_ = enter_CB;
    %% link(B,A)
    enter_BA = leave_CB_A;
    [n_BA, q_BA, leave_BA, emission_BA_L] = Link3_L(n_BA, q_BA, [enter_BA__, enter_BA_, enter_BA], green_time(1,k), cycle, [C_AD 1000*scale 1000*scale], Link_BA, scale);
    leave_BA_D = leave_BA(1); leave_BA_1=leave_BA(2); leave_BA_2=leave_BA(3);
    C_BA=C_BA-n_BA;
    enter_BA__=enter_BA_; enter_BA_=enter_BA;
    %% link(A,D)
    enter_AD = enter_AD_1+enter_AD_2+leave_BA_D;
    [n_AD, q_AD, leave_AD, emission_AD_L] = Link3_L(n_AD, q_AD, [enter_AD__, enter_AD_, enter_AD], cycle-green_time(3, k), cycle, [C_DE C_DG 1000*scale], Link_AD, scale);
    leave_AD_E = leave_AD(1); leave_AD_G = leave_AD(2); leave_AD_8 = leave_AD(3); 
    C_AD=C_AD-n_AD;
    enter_AD__ = enter_AD_;enter_AD_ = enter_AD;
    %% link(B,C)
    enter_BC = leave_AB_C;
    [n_BC, q_BC, leave_BC, emission_BC_L] = Link3_L(n_BC, q_BC, [enter_BC__, enter_BC_, enter_BC], green_time(2, k), cycle, [1000*scale, 1000*scale, C_CF], Link_BC, scale);
    leave_BC_3 = leave_BC(1); leave_BC_4 = leave_BC(2); leave_BC_F = leave_BC(3);
    C_BC=C_BC-n_BC;
    enter_BC__=enter_BC_; enter_BC_=enter_BC;
    %% link(C,F)
    enter_CF = enter_CF_3 + enter_CF_4 + leave_BC_F;
    [n_CF, q_CF, leave_CF, emission_CF_L] = Link2_L(n_CF, q_CF, [enter_CF__, enter_CF_, enter_CF], cycle-green_time(5, k), cycle, [C_FI, C_FE], Link_CF, scale);
    leave_CF_I = leave_CF(1); leave_CF_E = leave_CF(2);
    C_CF=C_CF-n_CF;
    enter_CF__ = enter_CF_; enter_CF_ = enter_CF;
    %% link(D,E)
    enter_DE = enter_DE_8 + leave_AD_E;
    [n_DE, q_DE, leave_DE, emission_DE_L] = Link2_L(n_DE, q_DE, [enter_DE__ enter_DE_ enter_DE], green_time(4, k), cycle, [C_EF, C_EH], Link_DE, scale);
    leave_DE_F = leave_DE(1); leave_DE_H = leave_DE(2);
    C_DE=C_DE-n_DE;
    enter_DE__ = enter_DE_; enter_DE_ = enter_DE;
    %% link(F,E)
    enter_FE = leave_CF_E;
    [n_FE, q_FE, leave_FE, emission_FE_L] = Link2_L(n_FE, q_FE, [enter_FE__ enter_FE_ enter_FE], green_time(4, k), cycle, [C_EH, C_ED], Link_FE, scale);
    leave_FE_H = leave_FE(1); leave_FE_D = leave_FE(2);
    C_FE=C_FE-n_FE;
    enter_FE__ = enter_FE_; enter_FE_ = enter_FE;
    %% link(B,E)
    enter_BE = leave_AB_E + leave_CB_E;
    [n_BE, q_BE, leave_BE, emission_BE_L] = Link3_L(n_BE, q_BE, [enter_BE__, enter_BE_, enter_BE], cycle-green_time(4, k), cycle, [C_EF, C_EH, C_ED], Link_BE, scale);
    leave_BE_F = leave_BE(1); leave_BE_H = leave_BE(2); leave_BE_D = leave_BE(3);
    C_BE=C_BE-n_BE;
    enter_BE__ = enter_BE_; enter_BE_ = enter_BE;
    %% link(E,D)
    enter_ED = leave_BE_D + leave_FE_D;
    [n_ED, q_ED, leave_ED, emission_ED_L] = Link2_L(n_ED, q_ED, [enter_ED__, enter_ED_, enter_ED], green_time(3, k), cycle, [C_DG, 1000*scale], Link_ED, scale);
    leave_ED_G = leave_ED(1); leave_ED_8 = leave_ED(2);
    C_ED=C_ED-n_ED;
    enter_ED__ = enter_ED_; enter_ED_ = enter_ED;
    %% link(D,G)
    enter_DG = enter_DG_8 + leave_AD_G + leave_ED_G;
    [n_DG, q_DG, leave_DG, emission_DG_L] = Link2_L(n_DG, q_DG, [enter_DG__, enter_DG_, enter_DG], cycle-green_time(6, k), cycle, [C_GH, 1000*scale], Link_DG, scale);
    leave_DG_H = leave_DG(1); leave_DG_7 = leave_DG(2);
    C_DG=C_DG-n_DG;
    enter_DG__ = enter_DG_; enter_DG_ = enter_DG;
    %% link(G,H)
    enter_GH = enter_GH_7 + leave_DG_H;
    [n_GH, q_GH, leave_GH, emission_GH_L] = Link2_L(n_GH, q_GH, [enter_GH__, enter_GH_, enter_GH], green_time(7, k), cycle, [C_HI, 1000*scale], Link_GH, scale);
    leave_GH_I = leave_GH(1); leave_GH_6 = leave_GH(2);
    C_GH=C_GH-n_GH;
    enter_GH__ = enter_GH_; enter_GH_ = enter_GH;
    %% link(E,F)
    enter_EF = leave_BE_F + leave_DE_F;
    [n_EF, q_EF, leave_EF, emission_EF_L] = Link1_L(n_EF, q_EF, [enter_EF__, enter_EF_, enter_EF], green_time(5, k), cycle, C_FI, Link_EF, scale);
    leave_EF_I = leave_EF;
    C_EF=C_EF-n_EF;
    enter_EF__ = enter_EF_; enter_EF_ = enter_EF;
    %% link(F,I)
    enter_FI = leave_CF_I + leave_EF_I;
    [n_FI, q_FI, leave_FI, emission_FI_L] = Link2_L(n_FI, q_FI, [enter_FI__ enter_FI_ enter_FI], cycle-green_time(8, k), cycle, [1000*scale C_IH], Link_FI, scale);
    leave_FI_5 = leave_FI(1); leave_FI_H = leave_FI(2);
    C_FI=C_FI-n_FI;
    enter_FI__ = enter_FI_; enter_FI_ = enter_FI;
    %% link(E,H)
    enter_EH = leave_DE_H + leave_BE_H + leave_FE_H;
    [n_EH, q_EH, leave_EH, emission_EH_L] = Link3_L(n_EH, q_EH, [enter_EH__ enter_EH_ enter_EH], cycle-green_time(7, k), cycle, [C_HI 1000*scale C_HG], Link_EH, scale);
    leave_EH_I = leave_EH(1); leave_EH_6 = leave_EH(2); leave_EH_G = leave_EH(3);
    C_EH=C_EH-n_EH;
    enter_EH__ = enter_EH_; enter_EH_ = enter_EH;
    %% link(I,H)
    enter_IH = enter_IH_5 + leave_FI_H;
    [n_IH, q_IH, leave_IH, emission_IH_L] = Link2_L(n_IH, q_IH, [enter_IH__ enter_IH_ enter_IH], green_time(7, k), cycle, [1000*scale C_HG], Link_IH, scale);
    leave_IH_6 = leave_IH(1); leave_IH_G = leave_IH(2);
    C_IH=C_IH-n_IH;
    enter_IH__ = enter_IH_; enter_IH_ = enter_IH;
    %% link(H,G)
    enter_HG = leave_EH_G + leave_IH_G + enter_HG_6;
    [n_HG, q_HG, leave_HG, emission_HG_L] = Link1_L(n_HG, q_HG, [enter_HG__ enter_HG_ enter_HG], green_time(6, k), cycle, 1000*scale, Link_HG, scale);
    leave_HG_7 = leave_HG;
    C_HG=C_HG-n_HG;
    enter_HG__ = enter_HG_; enter_HG_ = enter_HG;
    %% link(H,I)
    enter_HI = leave_EH_I + leave_GH_I + enter_HI_6;
    [n_HI, q_HI, leave_HI, emission_HI_L] = Link1_L(n_HI, q_HI, [enter_HI__ enter_HI_ enter_HI], green_time(8, k), cycle, 1000*scale, Link_HI, scale);
    leave_HI_5 = leave_HI;
    C_HI=C_HI-n_HI;
    enter_HI__ = enter_HI_; enter_HI_ = enter_HI;
    %% 
    Emission = emission_AB_L+emission_BA_L+emission_BC_L+emission_CB_L+emission_AD_L+emission_BE_L+emission_CF_L+emission_DE_L+emission_ED_L...
        +emission_EF_L+emission_FE_L+emission_DG_L+emission_EH_L+emission_FI_L+emission_GH_L+emission_HG_L+emission_HI_L+emission_IH_L...
        +emission_1A_L+ emission_2A_L+emission_3C_L+emission_4C_L+emission_5I_L+emission_6H_L+emission_7G_L+emission_8D_L;
    Emission_total_L(k) = Emission;
    Q_AB_L(k)=sum(q_AB);Q_BA_L(k)=sum(q_BA);Q_BC_L(k)=sum(q_BC);Q_CB_L(k)=sum(q_CB);Q_AD_L(k)=sum(q_AD);Q_BE_L(k)=sum(q_BE);
    Q_CF_L(k)=sum(q_CF);Q_DE_L(k)=sum(q_DE);Q_ED_L(k)=sum(q_ED);Q_EF_L(k)=sum(q_EF);Q_FE_L(k)=sum(q_FE);Q_DG_L(k)=sum(q_DG);
    Q_EH_L(k)=sum(q_EH);Q_FI_L(k)=sum(q_FI);Q_GH_L(k)=sum(q_GH);Q_HG_L(k)=sum(q_HG);Q_HI_L(k)=sum(q_HI);Q_IH_L(k)=sum(q_IH);
    Emission_AB_L(k) = emission_AB_L; Emission_BA_L(k) = emission_BA_L; Emission_BC_L(k) = emission_BC_L; Emission_CB_L(k) = emission_CB_L; Emission_AD_L(k) = emission_AD_L;
    Emission_BE_L(k) = emission_BE_L; Emission_CF_L(k) = emission_CF_L; Emission_DE_L(k) = emission_DE_L; Emission_ED_L(k) = emission_ED_L; Emission_EF_L(k) = emission_EF_L; 
    Emission_FE_L(k) = emission_FE_L; Emission_DG_L(k) = emission_DG_L; Emission_EH_L(k) = emission_EH_L; Emission_FI_L(k) = emission_FI_L; 
    Emission_GH_L(k) = emission_GH_L; Emission_HG_L(k) = emission_HG_L; Emission_HI_L(k) = emission_HI_L; Emission_IH_L(k) = emission_IH_L;
    n_total = n_AB + n_BA + n_BC + n_CB + n_AD + n_BE + n_CF + n_DE + n_ED + n_EF + n_FE + n_DG + n_EH + n_FI +...
        n_GH + n_HG + n_HI + n_IH+n_1A+n_2A+n_3C+n_4C+n_5I+n_6H+n_7G+n_8D;
    N_AB_L(k) = n_AB; N_BA_L(k) = n_BA; N_BC_L(k) = n_BC; N_CB_L(k) = n_CB; N_AD_L(k) = n_AD; N_BE_L(k) = n_BE; 
    N_CF_L(k) = n_CF; N_DE_L(k) = n_DE; N_ED_L(k) = n_ED; N_EF_L(k) = n_EF; N_FE_L(k) = n_FE; N_DG_L(k) = n_DG; 
    N_EH_L(k) = n_EH; N_FI_L(k) = n_FI; N_GH_L(k) = n_GH; N_HG_L(k) = n_HG; N_HI_L(k) = n_HI; N_IH_L(k) = n_IH;
    N_1A(k)=n_1A;N_2A(k)=n_2A;N_3C(k)=n_3C;N_4C(k)=n_4C;N_5I(k)=n_5I;N_6H(k)=n_6H;N_7G(k)=n_7G;
    N_8D(k)=n_8D;
    N_total_L(k) = n_total;
end
toc
%%
TTS = sum(N_total_L*cycle)/3600;
TE = sum(Emission_total_L); % total emission
fprintf('The total time spent is\n%.3f hours\n', TTS);
fprintf('The total emission of CO2 is\n%.3f kg\n', TE);

figure();
plot(1/(60/scale):1/(60/scale):N/(60/scale), N_total_L, '-*', 'linewidth', 1.5);
xlabel('step'); ylabel('vehicle');
xlim([0 N/(60/scale)]);
title('Total vehicles on the network every step');

figure();
plot(1:N, Emission_total_L, '*-', 'linewidth', 1.5);
xlabel('step'); ylabel('kg');
title('Total emissions of CO_2 on the network every step');

figure();
plot(1:N, Q_AB_L, 1:N, Q_BA_L, 1:N, Q_BC_L, 1:N, Q_CB_L, 1:N, Q_AD_L, 1:N, Q_BE_L, 1:N, Q_CF_L, 1:N, Q_DE_L, 1:N, Q_ED_L, 1:N, Q_EF_L, 1:N, Q_FE_L, 1:N, Q_DG_L...
    , 1:N, Q_EH_L, 1:N, Q_FI_L, 1:N, Q_GH_L, 1:N, Q_HG_L, 1:N, Q_HI_L, 1:N, Q_IH_L);
xlabel('step'); ylabel('vehicle');xlim([0 N]);
title('Queue length of every link');
legend('Q-AB','Q-BA','Q-BC','Q-CB','Q-AD','Q-BE','Q-CF','Q-DE','Q-ED','Q-EF','Q-FE','Q-DG','Q-EH','Q-FI','Q-GH','Q-HG','Q-HI','Q-IH');