clear;clc
close all
addpath('.\Source_oneday');
Random_Seed=clock;
Random_Seed=fix(Random_Seed(6)*1e7);
RandStream.setGlobalStream(RandStream('mt19937ar','seed',Random_Seed));
tic
Emission_day=80000;
Start_hour=7; End_hour=12;
Emission_real=0; % emission real during the whole day
scale = 5;  % time scale
% x0_L = zeros(80,1); % initial number of vehicles and queues are assumed to be zero
% x0_L = [x0_L;12*(Start_hour-1)+1;zeros(36,1);]; % the second item is the initial time step k=1
x0_L=[18.3240690875660,3.67235458668510,3.67235458668505,18.3240690875660,30.4315280698690,34.9288428039529,20.8894387598249,22.2773914662378,16.6197609951160,20.4818585385245,15.2157640349345,22.8360762024419,49.4057867987731,37.5883864809513,40.3918435830889,58.0404386655275,56.2006731003433,44.0713747134578,32.4848731785796,27.9166878878419,27.9166878878419,32.4848731785796,27.9166878878419,32.4848731785796,27.9166878878419,27.9166878878419,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,73,1190,1190,1190,1190,238,238,1356.60000000000,1356.60000000000,238,238,1356.60000000000,1356.60000000000,1092.64000000000,1092.64000000000,678.300000000000,678.300000000000,1904,1904,815.150000000000,815.150000000000,1228.89500000000,1228.89500000000,1591.55800000000,1591.55800000000,913.056000000000,913.056000000000,1591.35600000000,1591.35600000000,1946.73400000000,1946.73400000000,1736.54240000000,1736.54240000000,2286.96480000000,2286.96480000000,2214.47260000000,2214.47260000000]';
x_L = x0_L;
x_real = x_L; % independent of the rough state
x_real(81)=60*(Start_hour-1)+1; % the minute of the initial time
xx=[];U=[]; % collect the detailed states and inputs for the whole simulation
control_step_s=5; %control step for short-term system is 5 minutes
scenario=2;
pre_short=10; emission_record=[]; emission_con_record=[];
F_val=[]; f_val=[];

% control_step_L= 1hour;
for hour=Start_hour:End_hour
    Emission_con_L=Emission_day-Emission_real;
    %% Do the rough MPC for the remainder of the day, after every hour
    lb_L=10*scale; % lower bound for the greentime length
    ub_L=50*scale; % upper bound for the greentime length
    %% MPC controller
    Np_L=End_hour+1-hour; %prediction horizon is 24 hours; shrinking horzion; predicting the whole day is too long
    u_lb_L = lb_L*ones(8,Np_L); % lower bound of input
    u_ub_L = ub_L*ones(8,Np_L); % upper bound of input
    u0_L = (lb_L+ub_L)/2*ones(8,Np_L); % initial input for the first step, set variables best
    options = optimoptions(@fmincon,'Algorithm','sqp', 'TolFun',1e-2, 'TolX',1e-2, 'TolCon', 1e-2);
    % No receding horizon here, just predict once for one day
    u0_L=u0_L(:, 1:Np_L);
    objfun_L = @(u) Net_state_L(x_L,u,Np_L,scale,scenario);
    con_L = @(u) Emission_L(x_L,u,Np_L,scale,Emission_con_L,scenario);
    u_opt_L=u0_L; fval_opt=Net_state_L(x_L,u_opt_L,Np_L,scale,scenario); % benchmark
    if hour == 7
        N_max=500;
    elseif hour==8
        N_max=5;
    else
        N_max=5;
    end
%     N_max=5;
    for j=1:N_max % multi initial points
        if j>=2
            u0_L = randi([lb_L ub_L], 8, Np_L); % generate a random matrix consisting of integers between lb and ub
        end
        [u_L, fval, exitflag] = fmincon(objfun_L, u0_L, [], [], [], [], u_lb_L, u_ub_L, con_L, options);
        F_val=[F_val fval];
        count=0;
        while exitflag <= 0 && count<1
            warning('the solver does not succeed');
            u0_L=u_L;
            [u_L, fval, exitflag] = fmincon(objfun_L, u0_L, [], [], [], [], u_lb_L, u_ub_L, con_L, options);
            count=count+1;
        end
        if fval<=fval_opt
           fval_opt=fval;
           u_opt_L=u_L;
        end
    end
    ut_L = u_opt_L;
    u0_L = ut_L; % use the best solution as the initial point for the next step
%      ut=150.*ones(8,1);
    U_L=ut_L(:,1);hour % collect the input for next hour
    
    
    %% Simulate in the integrated detailed model for one hour to get the emissions for the next hour
    % Given the input for one hour, the initial state
    x_int=x_real; emission_hour_allow=NaN(60,1);
    x_int(81)=(hour-1)*60+1; % 81st element indicates the time step in minute
    % Expand the rough input to the detailed input by repeating every column 5 times
    u_int=repmat(U_L(:,1),1,60); 
    u_int=u_int/scale;
    for i=1:60 % 60 minute per hour
        [x_int, emission_min]=net_mpc_model(x_int,u_int(:,i),scenario);
        emission_hour_allow(i)=emission_min;
    end
    lb_s=10; % lower bound for the greentime length
    ub_s=50; % upper bound for the greentime length
    Np_s=pre_short/control_step_s; %prediction horizon
    u0_s = u_int(:,1:Np_s); % initial input for the first step, set variables best
    
    %% Adapter block: distribute the emission to every 10 minutes
    % initial value
    emission_ST_hat=NaN(60/pre_short,1);
    emission_real_hour=0; % emission real during this hour
    rho=0.1;
    for i=1:60/pre_short
        emission_ST_hat(i)=sum(emission_hour_allow(10*(i-1)+1:10*i));
    end
    
    x_s=[];U_s=[]; % initialized before every hour
    for i=1:60/pre_short % update the distribution every 10 minutes
        error_hour=sum(emission_hour_allow)-emission_real_hour-sum(emission_ST_hat);
        %% linear programming
        f=[-rho*error_hour*60/pre_short;1]; lb=[0;0];
        A=[error_hour*60/pre_short -1;
            -error_hour*60/pre_short -1];
        b=[error_hour; -error_hour];
        x_val=linprog(f,A,b,[],[],lb,[]);
        alpha=x_val(1);
        emission_ST_bar=emission_ST_hat+error_hour*alpha;
        emission_ST_next=emission_ST_bar(1); % this is for the next step emission (10 minutes)
        emission_con_record=[emission_con_record emission_ST_next];
        if max(size(emission_ST_bar))>1 % take the emission vector as the guess for the next iteration
            emission_ST_hat=emission_ST_bar(2:end);
        end
        %% Solve the short-term MPC problem for operation cycle
        options = optimoptions(@fmincon,'Algorithm','sqp','TolFun',1e-2, 'TolX',1e-2, 'TolCon', 1e-2);
        u_lb_s = lb_s*ones(8,Np_s); % lower bound of input
        u_ub_s = ub_s*ones(8,Np_s); % upper bound of input
%         u0_s=u0_s(:, 1:Np);
        objfun = @(u) Net_state(x_real,u,Np_s,control_step_s,scenario);
        con_s = @(u) Emission_s(x_real, u, Np_s, control_step_s, emission_ST_next,scenario);
        u_opt=u0_s; fval_opt=Net_state(x_real,u_opt,Np_s,control_step_s,scenario); % benchmark
        n_max=5;
        if hour==7
            if i < 3
                n_max=5;
            elseif i==3
                n_max=500;
            elseif i==4
                n_max=5;
            elseif i==5
                n_max=5;
            elseif i==6
                n_max=5;
            end
        else
            n_max=5;
        end

        for j=1:n_max % multi initial points
            if j==2
                u0_s = (lb_s+ub_s)/2*ones(8,Np_s);
            elseif j>2
                u0_s = randi([lb_s ub_s], 8, Np_s); % generate a random matrix consisting of integers between lb and ub
            end
            [u, fval, exitflag] = fmincon(objfun, u0_s, [], [], [], [], u_lb_s, u_ub_s, con_s, options);
            f_val=[f_val fval];
            count=0;
            while exitflag <= 0 && count<1
                u0_s=u;
                warning('the solver does not succeed');
                [u, fval, exitflag] = fmincon(objfun, u0_s, [], [], [], [], u_lb_s, u_ub_s, con_s, options);
                count=count+1;
            end
            if fval<=fval_opt
                fval_opt=fval;
                u_opt=u;
            end
        end
        u0_s=u_opt;
        % transform the control input to every minute
        ut_s=[];
        for i=1:size(u_opt,2)
            ut_s=[ut_s repmat(u_opt(:,i),1,control_step_s)];
        end
        % state evolution with the real system
        emission_real_shortstep=0;
        for i=1:pre_short
            [x_real, emis] = net_mpc_model(x_real, ut_s(:, i),scenario); % implement the first input, get the real state
            emission_real_hour=emission_real_hour+emis;
            emission_real_shortstep=emission_real_shortstep+emis;
            x_s=[x_s x_real]; % x_s stores the states for one hour
        end
        emission_record=[emission_record emission_real_shortstep];
        U_s=[U_s ut_s];
    end
    %% Transfer the short-term state into long-term state
    x_L(1:80)=x_real(1:80); % transfer the states to next iteration, at the last step
    x_L(81)=hour*12+1; % transfer the time step from short term into long term
    for i=1:18 % give average of the accumalated entering flow
        x_L(81+i*2-1)=sum(x_s(81+i*2-1,end-(2*scale-1):end-scale))/scale;
        x_L(81+i*2)=sum(x_s(81+i*2-1,end-(scale-1):end))/scale;
    end
    Emission_real=Emission_real+emission_real_hour;
    xx=[xx x_s]; U=[U U_s];
end
toc
%% Constraint function: long term
function [c, ceq]=Emission_L(x,u_hour,Np,scale,Emission_con_L,scenario)
ceq=0;
Np=Np*60/scale;u=[];
for i=1:size(u_hour,2)
    u=[u repmat(u_hour(:,i),1,12)];
end
Green_time = u; % transfer the input to green time length
xx=[]; yy=[]; 
    for p=1:Np
        green_time = Green_time(:,p); % allocate the input for current time step
        [x_, y] = net_mpc_model_L(x, green_time,scale,scenario); % predicted state evolution with given input
        xx=[xx x_]; % collect the state for the horizon of Np
        yy=[yy y]; % store the scalar value of the objective function for Np steps
        x=x_; % update current state
    end
    c=sum(yy)/1e4-Emission_con_L/1e4;
end
%% Constraint function: short term
function [c, ceq]=Emission_s(x, u_c, Np, control_step_s, emission_con_s,scenario)
ceq=0;
Np=Np*control_step_s; u=[];
for i=1:size(u_c,2)
    u=[u repmat(u_c(:,i),1,control_step_s)];
end
Green_time = u; % transfer the input to green time length
xx=[]; yy=[]; gama=0.05;
    for p=1:Np
        green_time = Green_time(:,p); % allocate the input for current time step
        [x_, y] = net_mpc_model(x, green_time,scenario); % predicted state evolution with given input
        xx=[xx x_]; % collect the state for the horizon of Np
        yy=[yy y]; % store the scalar value of the objective function for Np steps
        x=x_; % update current state
    end
    c=(sum(yy)-emission_con_s)/1e2;
end
%% Objective function: short term
function TTSE=Net_state(x, u_c, Np,control_step_s,scenario) % x gives the initial state at current step
Np=Np*control_step_s; u=[];
for i=1:size(u_c,2)
    u=[u repmat(u_c(:,i),1,control_step_s)];
end
Green_time = u; % transfer the input to green time length
xx=[]; yy=[]; gama=0.05;
    for p=1:Np
        green_time = Green_time(:,p); % allocate the input for current time step
        [x_, y] = net_mpc_model(x, green_time,scenario); % predicted state evolution with given input
        xx=[xx x_]; % collect the state for the horizon of Np
        yy=[yy y]; % store the scalar value of the objective function for Np steps
        x=x_; % update current state
    end
    xx=xx(1:26,:); % only count the number of vehicles on every link for every minute
    TTSE=sum(sum(xx))/60+gama*sum(yy); % Total vehicles
end
%% Objective function: long term
function TTSE=Net_state_L(x, u_hour, Np,scale,scenario) % x gives the initial state at current step
Np=Np*60/scale;u=[];
for i=1:size(u_hour,2)
    u=[u repmat(u_hour(:,i),1,60/scale)];
end
Green_time = u; % transfer the input to green time length
xx=[]; yy=[]; gama=0.05;
    for p=1:Np
        green_time = Green_time(:,p); % allocate the input for current time step
        [x_, y] = net_mpc_model_L(x, green_time,scale,scenario); % predicted state evolution with given input
        xx=[xx x_]; % collect the state for the horizon of Np
        yy=[yy y]; % store the scalar value of the objective function for Np steps
        x=x_; % update current state
    end
    xx=xx(1:26,:); % only count the number of vehicles on every link for every minute
    TTSE=sum(sum(xx))/(60/scale)+gama*sum(yy); % Total vehicles
end
%% Detailed model
function [x, y] = net_mpc_model(x,green_time,scenario)
 %% define the parameters of all the links
% parameter vector [link_length vehicle_lengt free_flow_speed turning_ratio[left straight right] saturating_flow[left straight right]]
% parameters=[5.6986,1824,45.27,621.7,1150.58,725.329,771.39,929,863,1217]';
parameters=[4.1357; 2354.2; 43.51; 671.61; 1104.24; 977.80; 887.358; 798.51; 809.71; 1029.577];
% parameters=[5 2000 60 700 1000 800 750 850 900 1200]'; % default value
l_veh=parameters(1); % default: 5m
sat=parameters(2); % saturating flow, default 2000
v_free=parameters(3); % default: 60km/h
v_idl=4;
l_1=parameters(4); % default: 700m
l_2=parameters(5);% default: 1000m
l_3=parameters(6); % default: 800m
l_4=parameters(7); % default: 750m
l_5=parameters(8); % default: 850m
l_6=parameters(9); % default: 900m
l_7=parameters(10); % default: 1200m
% parameter vector [link_length vehicle_lengt free_flow_speed low_speed turning_ratio[left straight right] saturating_flow[left straight right]]
Link_AB=[l_1, l_veh, v_free, v_idl, 2, [0.2 0.8], [sat, sat]]; %parameters of link(A,B), Link_2
Link_BA=[l_1, l_veh, v_free, v_idl, 3, [0.7 0.15 0.15], [sat, sat, sat]]; %parameters of link(A,B), Link_3, left turning ratio is small
Link_BC=[l_1, l_veh, v_free, v_idl, 3, [0.15 0.15 0.7], [sat, sat, sat]]; %parameters of link(B,C), Link_2
Link_CB=[l_1, l_veh, v_free, v_idl, 2, [0.8 0.2], [sat, sat]]; %parameters of link(A,B), Link_2
Link_AD=[l_3, l_veh, v_free, v_idl, 2, [0.4 0.2 0.4] [sat sat sat]];
Link_BE=[l_5, l_veh, v_free, v_idl, 3, [0.25 0.5 0.25], [sat sat sat]];
Link_CF=[l_1, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DE=[l_4, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]]; % Link_2
Link_ED=[l_4, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]]; %LInk_2
Link_EF=[l_3, l_veh, v_free, v_idl, 1, 1, sat]; %Link_1
Link_FE=[l_3, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DG=[l_6, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]];
Link_EH=[l_2, l_veh, v_free, v_idl, 3, [0.40 0.20 0.40], [sat sat sat]];
Link_FI=[l_7, l_veh, v_free, v_idl, 2, [0.6 0.4], [sat sat]];
Link_GH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_HG=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat]; %Link_1
Link_HI=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat];
Link_IH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_1A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_2A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_3C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_4C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_5I=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_6H=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_7G=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_8D=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];

C_AB=floor(Link_AB(1)*Link_AB(5)/Link_AB(2));
C_BA=floor(Link_BA(1)*Link_BA(5)/Link_BA(2));
C_BC=floor(Link_BC(1)*Link_BC(5)/Link_BC(2));
C_CB=floor(Link_CB(1)*Link_CB(5)/Link_CB(2));
C_AD=floor(Link_AD(1)*Link_AD(5)/Link_AD(2));
C_BE=floor(Link_BE(1)*Link_BE(5)/Link_BE(2));
C_CF=floor(Link_CF(1)*Link_CF(5)/Link_CF(2));
C_DE=floor(Link_DE(1)*Link_DE(5)/Link_DE(2));
C_ED=floor(Link_ED(1)*Link_ED(5)/Link_ED(2));
C_EF=floor(Link_EF(1)*Link_EF(5)/Link_EF(2));
C_FE=floor(Link_FE(1)*Link_FE(5)/Link_FE(2));
C_DG=floor(Link_DG(1)*Link_DG(5)/Link_DG(2));
C_EH=floor(Link_EH(1)*Link_EH(5)/Link_EH(2));
C_FI=floor(Link_FI(1)*Link_FI(5)/Link_FI(2));
C_GH=floor(Link_GH(1)*Link_GH(5)/Link_GH(2));
C_HG=floor(Link_HG(1)*Link_HG(5)/Link_HG(2));
C_HI=floor(Link_HI(1)*Link_HI(5)/Link_HI(2));
C_IH=floor(Link_IH(1)*Link_IH(5)/Link_IH(2));

n_AB=x(1);
n_BA=x(2);
n_BC=x(3);
n_CB=x(4);
n_AD=x(5);
n_BE=x(6);
n_CF=x(7);
n_DE=x(8);
n_ED=x(9);
n_EF=x(10);
n_FE=x(11);
n_DG=x(12);
n_EH=x(13);
n_FI=x(14);
n_GH=x(15);
n_HG=x(16);
n_HI=x(17);
n_IH=x(18);
n_1A=x(19);
n_2A=x(20);
n_3C=x(21);
n_4C=x(22);
n_5I=x(23);
n_6H=x(24);
n_7G=x(25);
n_8D=x(26);
q_AB=x(27:28);
q_BA=x(29:31);
q_BC=x(32:34);
q_CB=x(35:36);
q_AD=x(37:39);
q_BE=x(40:42);
q_CF=x(43:44);
q_DE=x(45:46);
q_ED=x(47:48);
q_EF=x(49);
q_FE=x(50:51);
q_DG=x(52:53);
q_EH=x(54:56);
q_FI=x(57:58);
q_GH=x(59:60);
q_HG=x(61);
q_HI=x(62);
q_IH=x(63:64);
q_1A=x(65:66);
q_2A=x(67:68);
q_3C=x(69:70);
q_4C=x(71:72);
q_5I=x(73:74);
q_6H=x(75:76);
q_7G=x(77:78);
q_8D=x(79:80);
k=x(81); % k is the time step, used to decide the current demands

enter_AB__=x(82);
enter_AB_=x(83);
enter_CB__=x(84);
enter_CB_=x(85);
enter_BA__=x(86);
enter_BA_=x(87);
enter_AD__=x(88);
enter_AD_=x(89);
enter_BC__=x(90);
enter_BC_=x(91);
enter_CF__=x(92);
enter_CF_=x(93);
enter_DE__=x(94);
enter_DE_=x(95);
enter_FE__=x(96);
enter_FE_=x(97);
enter_BE__=x(98);
enter_BE_=x(99);
enter_ED__=x(100);
enter_ED_=x(101);
enter_DG__=x(102);
enter_DG_=x(103);
enter_GH__=x(104);
enter_GH_=x(105);
enter_EF__=x(106);
enter_EF_=x(107);
enter_FI__=x(108);
enter_FI_=x(109);
enter_EH__=x(110);
enter_EH_=x(111);
enter_IH__=x(112);
enter_IH_=x(113);
enter_HG__=x(114);
enter_HG_=x(115);
enter_HI__=x(116);
enter_HI_=x(117);
cycle=60;
C_AB=C_AB-n_AB;
C_CB=C_CB-n_CB;
C_BA=C_BA-n_BA;
C_BA=C_BA-n_BA;
C_AD=C_AD-n_AD;
C_BC=C_BC-n_BC;
C_CF=C_CF-n_CF;
C_DE=C_DE-n_DE;
C_FE=C_FE-n_FE;
C_BE=C_BE-n_BE;
C_ED=C_ED-n_ED;
C_DG=C_DG-n_DG;
C_GH=C_GH-n_GH;
C_EF=C_EF-n_EF;
C_FI=C_FI-n_FI;
C_EH=C_EH-n_EH;
C_IH=C_IH-n_IH;
C_HG=C_HG-n_HG;
C_HI=C_HI-n_HI;
gama=0.3; % the weight of TTS and TE
    [enter_AB_1, enter_AD_1, n_1A, q_1A, emission_1A] = Link_S1nD(k, n_1A, q_1A, [C_AB, C_AD], green_time(1), cycle, Link_1A,scenario); % entering flow from the source
    [enter_AB_2, enter_AD_2, n_2A, q_2A, emission_2A] = Link_S2nD(k, n_2A, q_2A, [C_AB, C_AD], cycle-green_time(1), cycle, Link_2A,scenario);
    [enter_CF_3, enter_CB_3, n_3C, q_3C, emission_3C] = Link_S3nD(k, n_3C, q_3C, [C_CF, C_CB], cycle-green_time(2), cycle, Link_3C,scenario);
    [enter_CF_4, enter_CB_4, n_4C, q_4C, emission_4C] = Link_S4nD(k, n_4C, q_4C, [C_CF, C_CB], green_time(2), cycle, Link_4C,scenario);
    [enter_IH_5, n_5I, q_5I, emission_5I] = Link_S5nD(k, n_5I, q_5I, C_IH, green_time(8), cycle, Link_5I,scenario);
    [enter_HG_6, enter_HI_6, n_6H, q_6H, emission_6H] = Link_S6nD(k, n_6H, q_6H, [C_HG, C_HI], cycle-green_time(7), cycle, Link_6H,scenario);
    [enter_GH_7, n_7G, q_7G, emission_7G] = Link_S7nD(k, n_7G, q_7G, C_GH, cycle-green_time(6), cycle, Link_7G,scenario);
    [enter_DE_8, enter_DG_8, n_8D, q_8D, emission_8D] = Link_S8nD(k, n_8D, q_8D, [C_DE, C_DG], green_time(3), cycle, Link_8D,scenario);
    % simulate from the source links; leave is a vector: leaving to corresponding downstream links
    %% link(A,B), B does not need control
    enter_AB = enter_AB_1+enter_AB_2;
    [n_AB, q_AB, leave_AB, emission_AB] = Link2(n_AB, q_AB, [enter_AB__, enter_AB_, enter_AB], cycle, cycle, [C_BC C_BE], Link_AB);%update the state
    leave_AB_C = leave_AB(1); leave_AB_E = leave_AB(2); 
    enter_AB__= enter_AB_; enter_AB_= enter_AB; % update the enter of previous step, B does not need control
    %% link(C,B), B does not need control
    enter_CB = enter_CB_3 + enter_CB_4;
    [n_CB, q_CB, leave_CB, emission_CB] = Link2(n_CB, q_CB, [enter_CB__, enter_CB_, enter_CB], cycle, cycle, [C_BE, C_BA], Link_CB);
    leave_CB_E = leave_CB(1); leave_CB_A = leave_CB(2);
    enter_CB__ = enter_CB_; enter_CB_ = enter_CB;
    %% link(B,A)
    enter_BA = leave_CB_A;
    [n_BA, q_BA, leave_BA, emission_BA] = Link3(n_BA, q_BA, [enter_BA__, enter_BA_, enter_BA], green_time(1), cycle, [C_AD 1000 1000], Link_BA);
    leave_BA_D = leave_BA(1); leave_BA_1=leave_BA(2); leave_BA_2=leave_BA(3);
    enter_BA__=enter_BA_; enter_BA_=enter_BA;
    %% link(A,D)
    enter_AD = enter_AD_1+enter_AD_2+leave_BA_D;
    [n_AD, q_AD, leave_AD, emission_AD] = Link3(n_AD, q_AD, [enter_AD__, enter_AD_, enter_AD], cycle-green_time(3), cycle, [C_DE C_DG 1000], Link_AD);
    leave_AD_E = leave_AD(1); leave_AD_G = leave_AD(2); leave_AD_8 = leave_AD(3); 
    enter_AD__ = enter_AD_;enter_AD_ = enter_AD;
    %% link(B,C)
    enter_BC = leave_AB_C;
    [n_BC, q_BC, leave_BC, emission_BC] = Link3(n_BC, q_BC, [enter_BC__, enter_BC_, enter_BC], green_time(2), cycle, [1000, 1000, C_CF], Link_BC);
    leave_BC_3 = leave_BC(1); leave_BC_4 = leave_BC(2); leave_BC_F = leave_BC(3);
    enter_BC__=enter_BC_; enter_BC_=enter_BC;
    %% link(C,F)
    enter_CF = enter_CF_3 + enter_CF_4 + leave_BC_F;
    [n_CF, q_CF, leave_CF, emission_CF] = Link2(n_CF, q_CF, [enter_CF__, enter_CF_, enter_CF], cycle-green_time(5), cycle, [C_FI, C_FE], Link_CF);
    leave_CF_I = leave_CF(1); leave_CF_E = leave_CF(2);
    enter_CF__ = enter_CF_; enter_CF_ = enter_CF;
    %% link(D,E)
    enter_DE = enter_DE_8 + leave_AD_E;
    [n_DE, q_DE, leave_DE, emission_DE] = Link2(n_DE, q_DE, [enter_DE__ enter_DE_ enter_DE], green_time(4), cycle, [C_EF, C_EH], Link_DE);
    leave_DE_F = leave_DE(1); leave_DE_H = leave_DE(2);
    enter_DE__ = enter_DE_; enter_DE_ = enter_DE;
    %% link(F,E)
    enter_FE = leave_CF_E;
    [n_FE, q_FE, leave_FE, emission_FE] = Link2(n_FE, q_FE, [enter_FE__ enter_FE_ enter_FE], green_time(4), cycle, [C_EH, C_ED], Link_FE);
    leave_FE_H = leave_FE(1); leave_FE_D = leave_FE(2);
    enter_FE__ = enter_FE_; enter_FE_ = enter_FE;
    %% link(B,E)
    enter_BE = leave_AB_E + leave_CB_E;
    [n_BE, q_BE, leave_BE, emission_BE] = Link3(n_BE, q_BE, [enter_BE__, enter_BE_, enter_BE], cycle-green_time(4), cycle, [C_EF, C_EH, C_ED], Link_BE);
    leave_BE_F = leave_BE(1); leave_BE_H = leave_BE(2); leave_BE_D = leave_BE(3);
    enter_BE__ = enter_BE_; enter_BE_ = enter_BE;
    %% link(E,D)
    enter_ED = leave_BE_D + leave_FE_D;
    [n_ED, q_ED, leave_ED, emission_ED] = Link2(n_ED, q_ED, [enter_ED__, enter_ED_, enter_ED], green_time(3), cycle, [C_DG, 1000], Link_ED);
    leave_ED_G = leave_ED(1); leave_ED_8 = leave_ED(2);
    enter_ED__ = enter_ED_; enter_ED_ = enter_ED;
    %% link(D,G)
    enter_DG = enter_DG_8 + leave_AD_G + leave_ED_G;
    [n_DG, q_DG, leave_DG, emission_DG] = Link2(n_DG, q_DG, [enter_DG__, enter_DG_, enter_DG], cycle-green_time(6), cycle, [C_GH, 1000], Link_DG);
    leave_DG_H = leave_DG(1); leave_DG_7 = leave_DG(2);
    enter_DG__ = enter_DG_; enter_DG_ = enter_DG;
    %% link(G,H)
    enter_GH = enter_GH_7 + leave_DG_H;
    [n_GH, q_GH, leave_GH, emission_GH] = Link2(n_GH, q_GH, [enter_GH__, enter_GH_, enter_GH], green_time(7), cycle, [C_HI, 1000], Link_GH);
    leave_GH_I = leave_GH(1); leave_GH_6 = leave_GH(2);
    enter_GH__ = enter_GH_; enter_GH_ = enter_GH;
    %% link(E,F)
    enter_EF = leave_BE_F + leave_DE_F;
    [n_EF, q_EF, leave_EF, emission_EF] = Link1(n_EF, q_EF, [enter_EF__, enter_EF_, enter_EF], green_time(5), cycle, C_FI, Link_EF);
    leave_EF_I = leave_EF;
    enter_EF__ = enter_EF_; enter_EF_ = enter_EF;
    %% link(F,I)
    enter_FI = leave_CF_I + leave_EF_I;
    [n_FI, q_FI, leave_FI, emission_FI] = Link2(n_FI, q_FI, [enter_FI__ enter_FI_ enter_FI], cycle-green_time(8), cycle, [1000 C_IH], Link_FI);
    leave_FI_5 = leave_FI(1); leave_FI_H = leave_FI(2);
    enter_FI__ = enter_FI_; enter_FI_ = enter_FI;
    %% link(E,H)
    enter_EH = leave_DE_H + leave_BE_H + leave_FE_H;
    [n_EH, q_EH, leave_EH, emission_EH] = Link3(n_EH, q_EH, [enter_EH__ enter_EH_ enter_EH], cycle-green_time(7), cycle, [C_HI 1000 C_HG], Link_EH);
    leave_EH_I = leave_EH(1); leave_EH_6 = leave_EH(2); leave_EH_G = leave_EH(3);
    enter_EH__ = enter_EH_; enter_EH_ = enter_EH;
    %% link(I,H)
    enter_IH = enter_IH_5 + leave_FI_H;
    [n_IH, q_IH, leave_IH, emission_IH] = Link2(n_IH, q_IH, [enter_IH__ enter_IH_ enter_IH], green_time(7), cycle, [1000 C_HG], Link_IH);
    leave_IH_6 = leave_IH(1); leave_IH_G = leave_IH(2);
    enter_IH__ = enter_IH_; enter_IH_ = enter_IH;
    %% link(H,G)
    enter_HG = leave_EH_G + leave_IH_G + enter_HG_6;
    [n_HG, q_HG, leave_HG, emission_HG] = Link1(n_HG, q_HG, [enter_HG__ enter_HG_ enter_HG], green_time(6), cycle, 1000, Link_HG);
    leave_HG_7 = leave_HG;
    enter_HG__ = enter_HG_; enter_HG_ = enter_HG;
    %% link(H,I)
    enter_HI = leave_EH_I + leave_GH_I + enter_HI_6;
    [n_HI, q_HI, leave_HI, emission_HI] = Link1(n_HI, q_HI, [enter_HI__ enter_HI_ enter_HI], green_time(8), cycle, 1000, Link_HI);
    leave_HI_5 = leave_HI;
    enter_HI__ = enter_HI_; enter_HI_ = enter_HI;
    %% 
    k=k+1;
    x=[n_AB;n_BA;n_BC;n_CB;n_AD;n_BE;n_CF;n_DE;n_ED;n_EF;n_FE;n_DG;n_EH;n_FI;n_GH;n_HG;n_HI;n_IH; n_1A;n_2A;n_3C;...
        n_4C;n_5I;n_6H;n_7G;n_8D;q_AB;q_BA;q_BC;q_CB;q_AD;q_BE;q_CF;q_DE;q_ED;q_EF;q_FE;q_DG;q_EH;q_FI;q_GH;q_HG;q_HI;q_IH;...
        q_1A;q_2A;q_3C;q_4C;q_5I;q_6H;q_7G;q_8D;k;...
        enter_AB__;enter_AB_;enter_CB__;enter_CB_;enter_BA__;enter_BA_;enter_AD__;enter_AD_;enter_BC__;enter_BC_;...
        enter_CF__;enter_CF_;enter_DE__;enter_DE_;enter_FE__;enter_FE_;enter_BE__;enter_BE_;enter_ED__;enter_ED_;...
        enter_DG__;enter_DG_;enter_GH__;enter_GH_;enter_EF__;enter_EF_;enter_FI__;enter_FI_;enter_EH__;enter_EH_;...
        enter_IH__;enter_IH_;enter_HG__;enter_HG_;enter_HI__;enter_HI_];    
    y = emission_AB+emission_BA+emission_BC+...
        emission_CB+emission_AD+emission_BE+emission_CF+emission_DE+emission_ED+emission_EF+emission_FE+...
        emission_DG+emission_EH+emission_FI+emission_GH+emission_HG+emission_HI+emission_IH+emission_1A+...
        emission_2A+emission_3C+emission_4C+emission_5I+emission_6H+emission_7G+emission_8D;
end
%% Model for the long term
function [x, y] = net_mpc_model_L(x,green_time,scale,scenario)
 %% define the parameters of all the links
% parameter vector [link_length vehicle_lengt free_flow_speed turning_ratio[left straight right] saturating_flow[left straight right]]
% parameters=[5.6986,1824,45.27,621.7,1150.58,725.329,771.39,929,863,1217]';
parameters=[4.1357; 2354.2; 43.51; 671.61; 1104.24; 977.80; 887.358; 798.51; 809.71; 1029.577];
% parameters=[5 2000 60 700 1000 800 750 850 900 1200]'; % default value
l_veh=parameters(1); % default: 5m
sat=parameters(2); % saturating flow, default 2000
v_free=parameters(3); % default: 60km/h
v_idl=4;
l_1=parameters(4); % default: 700m
l_2=parameters(5);% default: 1000m
l_3=parameters(6); % default: 800m
l_4=parameters(7); % default: 750m
l_5=parameters(8); % default: 850m
l_6=parameters(9); % default: 900m
l_7=parameters(10); % default: 1200m
% parameter vector [link_length vehicle_lengt free_flow_speed low_speed turning_ratio[left straight right] saturating_flow[left straight right]]
Link_AB=[l_1, l_veh, v_free, v_idl, 2, [0.2 0.8], [sat, sat]]; %parameters of link(A,B), Link_2
Link_BA=[l_1, l_veh, v_free, v_idl, 3, [0.7 0.15 0.15], [sat, sat, sat]]; %parameters of link(A,B), Link_3, left turning ratio is small
Link_BC=[l_1, l_veh, v_free, v_idl, 3, [0.15 0.15 0.7], [sat, sat, sat]]; %parameters of link(B,C), Link_2
Link_CB=[l_1, l_veh, v_free, v_idl, 2, [0.8 0.2], [sat, sat]]; %parameters of link(A,B), Link_2
Link_AD=[l_3, l_veh, v_free, v_idl, 2, [0.4 0.2 0.4] [sat sat sat]];
Link_BE=[l_5, l_veh, v_free, v_idl, 3, [0.25 0.5 0.25], [sat sat sat]];
Link_CF=[l_1, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DE=[l_4, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]]; % Link_2
Link_ED=[l_4, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]]; %LInk_2
Link_EF=[l_3, l_veh, v_free, v_idl, 1, 1, sat]; %Link_1
Link_FE=[l_3, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_DG=[l_6, l_veh, v_free, v_idl, 2, [0.4 0.6], [sat sat]];
Link_EH=[l_2, l_veh, v_free, v_idl, 3, [0.40 0.20 0.40], [sat sat sat]];
Link_FI=[l_7, l_veh, v_free, v_idl, 2, [0.6 0.4], [sat sat]];
Link_GH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_HG=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat]; %Link_1
Link_HI=[l_2, l_veh, v_free, v_idl, 2, 1, 2*sat];
Link_IH=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_1A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_2A=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_3C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_4C=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_5I=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_6H=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_7G=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
Link_8D=[l_2, l_veh, v_free, v_idl, 2, [0.5 0.5], [sat sat]];
scale1 = scale;
C_AB=floor(Link_AB(1)*Link_AB(5)/Link_AB(2))*scale1;
C_BA=floor(Link_BA(1)*Link_BA(5)/Link_BA(2))*scale1;
C_BC=floor(Link_BC(1)*Link_BC(5)/Link_BC(2))*scale1;
C_CB=floor(Link_CB(1)*Link_CB(5)/Link_CB(2))*scale1;
C_AD=floor(Link_AD(1)*Link_AD(5)/Link_AD(2))*scale1;
C_BE=floor(Link_BE(1)*Link_BE(5)/Link_BE(2))*scale1;
C_CF=floor(Link_CF(1)*Link_CF(5)/Link_CF(2))*scale1;
C_DE=floor(Link_DE(1)*Link_DE(5)/Link_DE(2))*scale1;
C_ED=floor(Link_ED(1)*Link_ED(5)/Link_ED(2))*scale1;
C_EF=floor(Link_EF(1)*Link_EF(5)/Link_EF(2))*scale1;
C_FE=floor(Link_FE(1)*Link_FE(5)/Link_FE(2))*scale1;
C_DG=floor(Link_DG(1)*Link_DG(5)/Link_DG(2))*scale1;
C_EH=floor(Link_EH(1)*Link_EH(5)/Link_EH(2))*scale1;
C_FI=floor(Link_FI(1)*Link_FI(5)/Link_FI(2))*scale1;
C_GH=floor(Link_GH(1)*Link_GH(5)/Link_GH(2))*scale1;
C_HG=floor(Link_HG(1)*Link_HG(5)/Link_HG(2))*scale1;
C_HI=floor(Link_HI(1)*Link_HI(5)/Link_HI(2))*scale1;
C_IH=floor(Link_IH(1)*Link_IH(5)/Link_IH(2))*scale1;

n_AB=x(1);
n_BA=x(2);
n_BC=x(3);
n_CB=x(4);
n_AD=x(5);
n_BE=x(6);
n_CF=x(7);
n_DE=x(8);
n_ED=x(9);
n_EF=x(10);
n_FE=x(11);
n_DG=x(12);
n_EH=x(13);
n_FI=x(14);
n_GH=x(15);
n_HG=x(16);
n_HI=x(17);
n_IH=x(18);
n_1A=x(19);
n_2A=x(20);
n_3C=x(21);
n_4C=x(22);
n_5I=x(23);
n_6H=x(24);
n_7G=x(25);
n_8D=x(26);
q_AB=x(27:28);
q_BA=x(29:31);
q_BC=x(32:34);
q_CB=x(35:36);
q_AD=x(37:39);
q_BE=x(40:42);
q_CF=x(43:44);
q_DE=x(45:46);
q_ED=x(47:48);
q_EF=x(49);
q_FE=x(50:51);
q_DG=x(52:53);
q_EH=x(54:56);
q_FI=x(57:58);
q_GH=x(59:60);
q_HG=x(61);
q_HI=x(62);
q_IH=x(63:64);
q_1A=x(65:66);
q_2A=x(67:68);
q_3C=x(69:70);
q_4C=x(71:72);
q_5I=x(73:74);
q_6H=x(75:76);
q_7G=x(77:78);
q_8D=x(79:80);
k=x(81); % k is the time step, used to decide the current demands

enter_AB__=x(82);
enter_AB_=x(83);
enter_CB__=x(84);
enter_CB_=x(85);
enter_BA__=x(86);
enter_BA_=x(87);
enter_AD__=x(88);
enter_AD_=x(89);
enter_BC__=x(90);
enter_BC_=x(91);
enter_CF__=x(92);
enter_CF_=x(93);
enter_DE__=x(94);
enter_DE_=x(95);
enter_FE__=x(96);
enter_FE_=x(97);
enter_BE__=x(98);
enter_BE_=x(99);
enter_ED__=x(100);
enter_ED_=x(101);
enter_DG__=x(102);
enter_DG_=x(103);
enter_GH__=x(104);
enter_GH_=x(105);
enter_EF__=x(106);
enter_EF_=x(107);
enter_FI__=x(108);
enter_FI_=x(109);
enter_EH__=x(110);
enter_EH_=x(111);
enter_IH__=x(112);
enter_IH_=x(113);
enter_HG__=x(114);
enter_HG_=x(115);
enter_HI__=x(116);
enter_HI_=x(117);
cycle=60*scale;
gama=0.3; % the weight of TTS and TE
    [enter_AB_1, enter_AD_1, n_1A, q_1A, emission_1A_L] = Link_S1nLD(k, n_1A, q_1A, [C_AB, C_AD], green_time(1), cycle, Link_1A, scale,scenario); % entering flow from the source
    [enter_AB_2, enter_AD_2, n_2A, q_2A, emission_2A_L] = Link_S2nLD(k, n_2A, q_2A, [C_AB, C_AD], cycle-green_time(1), cycle, Link_2A, scale,scenario);
    [enter_CF_3, enter_CB_3, n_3C, q_3C, emission_3C_L] = Link_S3nLD(k, n_3C, q_3C, [C_CF, C_CB], cycle-green_time(2), cycle, Link_3C, scale,scenario);
    [enter_CF_4, enter_CB_4, n_4C, q_4C, emission_4C_L] = Link_S4nLD(k, n_4C, q_4C, [C_CF, C_CB], green_time(2), cycle, Link_4C, scale,scenario);
    [enter_IH_5, n_5I, q_5I, emission_5I_L] = Link_S5nLD(k, n_5I, q_5I, C_IH, green_time(8), cycle, Link_5I, scale,scenario);
    [enter_HG_6, enter_HI_6, n_6H, q_6H, emission_6H_L] = Link_S6nLD(k, n_6H, q_6H, [C_HG, C_HI], cycle-green_time(7), cycle, Link_6H, scale,scenario);
    [enter_GH_7, n_7G, q_7G, emission_7G_L] = Link_S7nLD(k, n_7G, q_7G, C_GH, cycle-green_time(6), cycle, Link_7G, scale,scenario);
    [enter_DE_8, enter_DG_8, n_8D, q_8D, emission_8D_L] = Link_S8nLD(k, n_8D, q_8D, [C_DE, C_DG], green_time(3), cycle, Link_8D, scale,scenario);
    % simulate from the source links; leave is a vector: leaving to corresponding downstream links
    %% link(A,B), B does not need control
    enter_AB = enter_AB_1+enter_AB_2;
    [n_AB, q_AB, leave_AB, emission_AB_L] = Link2_L(n_AB, q_AB, [enter_AB__, enter_AB_, enter_AB], cycle, cycle, [C_BC C_BE], Link_AB, scale);%update the state
    leave_AB_C = leave_AB(1); leave_AB_E = leave_AB(2); 
    C_AB=C_AB-n_AB;
    enter_AB__= enter_AB_; enter_AB_= enter_AB; % update the enter of previous step, B does not need control
    %% link(C,B), B does not need control
    enter_CB = enter_CB_3 + enter_CB_4;
    [n_CB, q_CB, leave_CB, emission_CB_L] = Link2_L(n_CB, q_CB, [enter_CB__, enter_CB_, enter_CB], cycle, cycle, [C_BE, C_BA], Link_CB, scale);
    leave_CB_E = leave_CB(1); leave_CB_A = leave_CB(2);
    C_CB=C_CB-n_CB;
    enter_CB__ = enter_CB_; enter_CB_ = enter_CB;
    %% link(B,A)
    enter_BA = leave_CB_A;
    [n_BA, q_BA, leave_BA, emission_BA_L] = Link3_L(n_BA, q_BA, [enter_BA__, enter_BA_, enter_BA], green_time(1), cycle, [C_AD 1000*scale 1000*scale], Link_BA, scale);
    leave_BA_D = leave_BA(1); leave_BA_1=leave_BA(2); leave_BA_2=leave_BA(3);
    C_BA=C_BA-n_BA;
    enter_BA__=enter_BA_; enter_BA_=enter_BA;
    %% link(A,D)
    enter_AD = enter_AD_1+enter_AD_2+leave_BA_D;
    [n_AD, q_AD, leave_AD, emission_AD_L] = Link3_L(n_AD, q_AD, [enter_AD__, enter_AD_, enter_AD], cycle-green_time(3), cycle, [C_DE C_DG 1000*scale], Link_AD, scale);
    leave_AD_E = leave_AD(1); leave_AD_G = leave_AD(2); leave_AD_8 = leave_AD(3); 
    C_AD=C_AD-n_AD;
    enter_AD__ = enter_AD_;enter_AD_ = enter_AD;
    %% link(B,C)
    enter_BC = leave_AB_C;
    [n_BC, q_BC, leave_BC, emission_BC_L] = Link3_L(n_BC, q_BC, [enter_BC__, enter_BC_, enter_BC], green_time(2), cycle, [1000*scale, 1000*scale, C_CF], Link_BC, scale);
    leave_BC_3 = leave_BC(1); leave_BC_4 = leave_BC(2); leave_BC_F = leave_BC(3);
    C_BC=C_BC-n_BC;
    enter_BC__=enter_BC_; enter_BC_=enter_BC;
    %% link(C,F)
    enter_CF = enter_CF_3 + enter_CF_4 + leave_BC_F;
    [n_CF, q_CF, leave_CF, emission_CF_L] = Link2_L(n_CF, q_CF, [enter_CF__, enter_CF_, enter_CF], cycle-green_time(5), cycle, [C_FI, C_FE], Link_CF, scale);
    leave_CF_I = leave_CF(1); leave_CF_E = leave_CF(2);
    C_CF=C_CF-n_CF;
    enter_CF__ = enter_CF_; enter_CF_ = enter_CF;
    %% link(D,E)
    enter_DE = enter_DE_8 + leave_AD_E;
    [n_DE, q_DE, leave_DE, emission_DE_L] = Link2_L(n_DE, q_DE, [enter_DE__ enter_DE_ enter_DE], green_time(4), cycle, [C_EF, C_EH], Link_DE, scale);
    leave_DE_F = leave_DE(1); leave_DE_H = leave_DE(2);
    C_DE=C_DE-n_DE;
    enter_DE__ = enter_DE_; enter_DE_ = enter_DE;
    %% link(F,E)
    enter_FE = leave_CF_E;
    [n_FE, q_FE, leave_FE, emission_FE_L] = Link2_L(n_FE, q_FE, [enter_FE__ enter_FE_ enter_FE], green_time(4), cycle, [C_EH, C_ED], Link_FE, scale);
    leave_FE_H = leave_FE(1); leave_FE_D = leave_FE(2);
    C_FE=C_FE-n_FE;
    enter_FE__ = enter_FE_; enter_FE_ = enter_FE;
    %% link(B,E)
    enter_BE = leave_AB_E + leave_CB_E;
    [n_BE, q_BE, leave_BE, emission_BE_L] = Link3_L(n_BE, q_BE, [enter_BE__, enter_BE_, enter_BE], cycle-green_time(4), cycle, [C_EF, C_EH, C_ED], Link_BE, scale);
    leave_BE_F = leave_BE(1); leave_BE_H = leave_BE(2); leave_BE_D = leave_BE(3);
    C_BE=C_BE-n_BE;
    enter_BE__ = enter_BE_; enter_BE_ = enter_BE;
    %% link(E,D)
    enter_ED = leave_BE_D + leave_FE_D;
    [n_ED, q_ED, leave_ED, emission_ED_L] = Link2_L(n_ED, q_ED, [enter_ED__, enter_ED_, enter_ED], green_time(3), cycle, [C_DG, 1000*scale], Link_ED, scale);
    leave_ED_G = leave_ED(1); leave_ED_8 = leave_ED(2);
    C_ED=C_ED-n_ED;
    enter_ED__ = enter_ED_; enter_ED_ = enter_ED;
    %% link(D,G)
    enter_DG = enter_DG_8 + leave_AD_G + leave_ED_G;
    [n_DG, q_DG, leave_DG, emission_DG_L] = Link2_L(n_DG, q_DG, [enter_DG__, enter_DG_, enter_DG], cycle-green_time(6), cycle, [C_GH, 1000*scale], Link_DG, scale);
    leave_DG_H = leave_DG(1); leave_DG_7 = leave_DG(2);
    C_DG=C_DG-n_DG;
    enter_DG__ = enter_DG_; enter_DG_ = enter_DG;
    %% link(G,H)
    enter_GH = enter_GH_7 + leave_DG_H;
    [n_GH, q_GH, leave_GH, emission_GH_L] = Link2_L(n_GH, q_GH, [enter_GH__, enter_GH_, enter_GH], green_time(7), cycle, [C_HI, 1000*scale], Link_GH, scale);
    leave_GH_I = leave_GH(1); leave_GH_6 = leave_GH(2);
    C_GH=C_GH-n_GH;
    enter_GH__ = enter_GH_; enter_GH_ = enter_GH;
    %% link(E,F)
    enter_EF = leave_BE_F + leave_DE_F;
    [n_EF, q_EF, leave_EF, emission_EF_L] = Link1_L(n_EF, q_EF, [enter_EF__, enter_EF_, enter_EF], green_time(5), cycle, C_FI, Link_EF, scale);
    leave_EF_I = leave_EF;
    C_EF=C_EF-n_EF;
    enter_EF__ = enter_EF_; enter_EF_ = enter_EF;
    %% link(F,I)
    enter_FI = leave_CF_I + leave_EF_I;
    [n_FI, q_FI, leave_FI, emission_FI_L] = Link2_L(n_FI, q_FI, [enter_FI__ enter_FI_ enter_FI], cycle-green_time(8), cycle, [1000*scale C_IH], Link_FI, scale);
    leave_FI_5 = leave_FI(1); leave_FI_H = leave_FI(2);
    C_FI=C_FI-n_FI;
    enter_FI__ = enter_FI_; enter_FI_ = enter_FI;
    %% link(E,H)
    enter_EH = leave_DE_H + leave_BE_H + leave_FE_H;
    [n_EH, q_EH, leave_EH, emission_EH_L] = Link3_L(n_EH, q_EH, [enter_EH__ enter_EH_ enter_EH], cycle-green_time(7), cycle, [C_HI 1000*scale C_HG], Link_EH, scale);
    leave_EH_I = leave_EH(1); leave_EH_6 = leave_EH(2); leave_EH_G = leave_EH(3);
    C_EH=C_EH-n_EH;
    enter_EH__ = enter_EH_; enter_EH_ = enter_EH;
    %% link(I,H)
    enter_IH = enter_IH_5 + leave_FI_H;
    [n_IH, q_IH, leave_IH, emission_IH_L] = Link2_L(n_IH, q_IH, [enter_IH__ enter_IH_ enter_IH], green_time(7), cycle, [1000*scale C_HG], Link_IH, scale);
    leave_IH_6 = leave_IH(1); leave_IH_G = leave_IH(2);
    C_IH=C_IH-n_IH;
    enter_IH__ = enter_IH_; enter_IH_ = enter_IH;
    %% link(H,G)
    enter_HG = leave_EH_G + leave_IH_G + enter_HG_6;
    [n_HG, q_HG, leave_HG, emission_HG_L] = Link1_L(n_HG, q_HG, [enter_HG__ enter_HG_ enter_HG], green_time(6), cycle, 1000*scale, Link_HG, scale);
    leave_HG_7 = leave_HG;
    C_HG=C_HG-n_HG;
    enter_HG__ = enter_HG_; enter_HG_ = enter_HG;
    %% link(H,I)
    enter_HI = leave_EH_I + leave_GH_I + enter_HI_6;
    [n_HI, q_HI, leave_HI, emission_HI_L] = Link1_L(n_HI, q_HI, [enter_HI__ enter_HI_ enter_HI], green_time(8), cycle, 1000*scale, Link_HI, scale);
    leave_HI_5 = leave_HI;
    C_HI=C_HI-n_HI;
    enter_HI__ = enter_HI_; enter_HI_ = enter_HI;
    %% 
    k=k+1;
    emission_sum=emission_AB_L+emission_BA_L+emission_BC_L+...
        emission_CB_L+emission_AD_L+emission_BE_L+emission_CF_L+emission_DE_L+emission_ED_L+emission_EF_L+emission_FE_L+...
        emission_DG_L+emission_EH_L+emission_FI_L+emission_GH_L+emission_HG_L+emission_HI_L+emission_IH_L...
        +emission_1A_L+ emission_2A_L+emission_3C_L+emission_4C_L+emission_5I_L+emission_6H_L+emission_7G_L+emission_8D_L;
    x=[n_AB;n_BA;n_BC;n_CB;n_AD;n_BE;n_CF;n_DE;n_ED;n_EF;n_FE;n_DG;n_EH;n_FI;n_GH;n_HG;n_HI;n_IH; n_1A;n_2A;n_3C;...
        n_4C;n_5I;n_6H;n_7G;n_8D;q_AB;q_BA;q_BC;q_CB;q_AD;q_BE;q_CF;q_DE;q_ED;q_EF;q_FE;q_DG;q_EH;q_FI;q_GH;q_HG;q_HI;q_IH;...
        q_1A;q_2A;q_3C;q_4C;q_5I;q_6H;q_7G;q_8D;k;...
        enter_AB__;enter_AB_;enter_CB__;enter_CB_;enter_BA__;enter_BA_;enter_AD__;enter_AD_;enter_BC__;enter_BC_;...
        enter_CF__;enter_CF_;enter_DE__;enter_DE_;enter_FE__;enter_FE_;enter_BE__;enter_BE_;enter_ED__;enter_ED_;...
        enter_DG__;enter_DG_;enter_GH__;enter_GH_;enter_EF__;enter_EF_;enter_FI__;enter_FI_;enter_EH__;enter_EH_;...
        enter_IH__;enter_IH_;enter_HG__;enter_HG_;enter_HI__;enter_HI_];    
    
    y = emission_sum;
end